
/* Diagram styles */
.pinout-diagram{
    font-family:Verdana, Geneva, sans-serif;  
}

{% if legend %}
/* Label styles */
{% for entry in legend %}
.{{ entry[1] }} .label_box{ fill: {{ entry[2] }} }
.leaderline_{{ entry[1] }} { stroke:   {{ entry[2] }} }
{% endfor %}
{% endif %}


.label text{
    fill: {{ label_text_color }};
    font-family:Verdana, Geneva, san-serif; 
    font-size: {{ label_font_size }}px; 
    font-weight: bold;
}


/* Legend */
.legend{
    font-size: {{ legend_font_size }}px; 
}

.legend_box{
    fill: rgb(247, 247, 247);
    stroke: rgb(204, 204, 204);
}