/*
    # Diagram styling 
*/

.pinout-diagram{
    font-family:verdana, san-serif; 
    margin: 20px auto;   
}


/* 
    # Label styles
*/

/* Default label styles - applied to labels with no tags specified */
.label_box{ fill: rgb(63, 63, 63) }
.leaderline{ stroke:   rgb(63, 63, 63) }

/* Labels styled by their tags */
.gpio .label_box{ fill: rgb(194, 100, 0); }
.gpio .leaderline{ stroke:   rgb(194, 100, 0); }
.leaderline_gpio{ stroke:   rgb(194, 100, 0); }

.analog .label_box{ fill: rgb(115, 85, 130); }
.analog .leaderline{ stroke:   rgb(115, 85, 130); }
.leaderline_analog{ stroke:   rgb(115, 85, 130); }

.pwm .label_box{ fill: rgb(3, 86, 150); }
.pwm .leaderline{ stroke:   rgb(3, 86, 150); }
.leaderline_pwm{ stroke:   rgb(3, 86, 150); }

.pwr .label_box{ fill: rgb(151, 28, 0); }
.pwr .leaderline{ stroke:   rgb(151, 28, 0); }
.leaderline_pwr{ stroke:   rgb(151, 28, 0); }


.label text{
    fill: #fff;
    font-family:verdana, san-serif; 
    font-size: 11px; 
    font-weight: bold;
}


/* Legend */
.legend{
    font-size: 13px; 
}

.legend_box{
    fill: rgb(247, 247, 247);
    stroke: rgb(225, 225, 225);
}